#!/bin/bash
#
# Yocto build script for the MitySOM-AM57X using a custom layer
# Expected to be ran from within crops/poky:ubuntu-18.04 docker /work directory
set -e

###############################################################################
#			CUSTOMIZABLE PARAMETERS
###############################################################################
DEPLOY_DIR=${PWD}/deploy
CUSTOM_LAYER=${PWD##*/}
# Leave this blank to auto-detect, set manually if more than one recipe exists
CUSTOM_IMAGE_RECIPE=""

###############################################################################
#			DO NOT MODIFY BELOW THIS
###############################################################################

# Try to get custom image name
if [ -z ${CUSTOM_IMAGE_RECIPE} ]; then
	CUSTOM_IMAGE_RECIPE=$(find recipes/images -iname "*.bb")
	if [ $(echo ${CUSTOM_IMAGE_RECIPE} | wc -l) -gt 1 ]; then
		echo "More than one image recipe found. Manually set CUSTOM_IMAGE_RECIPE"
		exit 1
	elif [ $(echo ${CUSTOM_IMAGE_RECIPE} | wc -l) -ne 1 ]; then
		echo "No image recipe found"
		exit 1
	fi
fi
CUSTOM_IMAGE=$(basename ${CUSTOM_IMAGE_RECIPE} | cut -d "." -f1)

# Try to get custom device tree
DEVICE_TREE_FILE=$(find recipes-kernel -iname "*.dts")
if [ $(echo ${DEVICE_TREE_FILE} | wc -l) -ne 1 ]; then
	echo "No device tree file found"
else
	DEVICE_TREE=$(basename ${DEVICE_TREE_FILE} | cut -d "." -f1)
fi

# Try to get uEnv.txt file
UENV_FILE=$(find recipes-bsp/u-boot -iname "uEnv.txt")

# Get list of files to copy from custom meta-layer
CUSTOM_LAYER_FILES=$(ls)

# Download and install toolchain
TOOLCHAIN_DIR=/work/gcc-arm-8.3-2019.03-x86_64-arm-linux-gnueabihf/
if [ ! -d "${TOOLCHAIN_DIR}"  ]; then
	    echo "${TOOLCHAIN_DIR} does not exist. Redownload and install"
	        wget https://developer.arm.com/-/media/Files/downloads/gnu-a/8.3-2019.03/binrel/gcc-arm-8.3-2019.03-x86_64-arm-linux-gnueabihf.tar.xz -O - | tar -Jx -C /work
fi
export TOOLCHAIN_PATH_ARMV7=/work/gcc-arm-8.3-2019.03-x86_64-arm-linux-gnueabihf

# Check if oe-layersetup exists
# If so, assume that yocto has been setup
if [ ! -d oe-layersetup ]; then
	# Create file list to copy custom layer where it is needed
	# Checkout the oe-layersetup tools
	git clone http://support.criticallink.com/git/oe-layersetup.git --branch criticallink
	pushd oe-layersetup
	./oe-layertool-setup.sh -f configs/processor-sdk/processor-sdk-06.03.00.106-cl-external-config.txt
	popd
	# Copy the custom meta layer where it's needed for the build
	mkdir -p oe-layersetup/sources/${CUSTOM_LAYER}
	cp -r ${CUSTOM_LAYER_FILES} oe-layersetup/sources/${CUSTOM_LAYER}/
	pushd oe-layersetup/build
	source conf/setenv
	MACHINE=mitysom-am57x bitbake-layers add-layer ../sources/${CUSTOM_LAYER}
	echo 'INHERIT           += " rm_work" '	>> conf/local.conf
	echo 'INHERIT           += " archiver"'	>> conf/local.conf
	echo 'ARCHIVER_MODE[src] = "original"' 	>> conf/local.conf
	if [ ! -z ${DEVICE_TREE} ]; then
		echo 'KERNEL_DEVICETREE  = "${DEVICE_TREE}.dtb"' >> conf/local.conf
	fi
	if [ ! -z ${UENV_FILE} ]; then
		echo 'IMAGE_BOOT_FILES  += "uEnv.txt"' >> conf/local.conf
	fi
else
	pushd oe-layersetup/build
	source conf/setenv
fi

# Build custom image
MACHINE=mitysom-am57x bitbake ${CUSTOM_IMAGE}

# Clear deploy directory
rm -rf ${DEPLOY_DIR}
mkdir -p ${DEPLOY_DIR}

# Deploy artifacts
export BUILDDIR=${PWD}
pushd arago-tmp-external-arm-toolchain/deploy/images/mitysom-am57x/
cp u-boot.img			${DEPLOY_DIR}/
cp MLO				${DEPLOY_DIR}/
cp uEnv.txt			${DEPLOY_DIR}/
# Deploy SPL image with usb boot support
cp u-boot-spl.bin		${DEPLOY_DIR}/
# Deploy kernel and modules
cp zImage 			${DEPLOY_DIR}/
cp modules-mitysom-am57x.tgz 	${DEPLOY_DIR}/
# Deploy device tree
cp *mity*dtb 			${DEPLOY_DIR}/
# Rootfs
cp *rootfs*.tar.xz 		${DEPLOY_DIR}/
cp *.manifest 			${DEPLOY_DIR}/
# Rename .wic to .img and package sdcard images into zip files
# Script is deployed by recipe zip-wic-images.bb
./zip-wic-images.sh
# Move zipped images to DEPLOY_DIR
for f in *.img.zip; do
	mv "$f" "${DEPLOY_DIR}/"
done
